//
// Copyright (c) 2019 muukii
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
import Foundation

/// Dictionary based index storage
public struct HashIndex<Schema: EntitySchemaType, HashKey: Hashable, Entity: EntityType>: IndexType, Equatable {
  
  // FIXME: To be faster filter, use BTree
  private var backing: [HashKey : AnyEntityIdentifier] = [:]
  
  public init() {
  }
  
  public mutating func _apply(removing: Set<AnyEntityIdentifier>, entityName: EntityTableIdentifier) {
    
    if Entity.entityName == entityName {
      
      var keys: Set<HashKey> = .init()
      backing.forEach { key, value in
        if removing.contains(value) {
          keys.insert(key)
        }
      }
      
      keys.forEach {
        backing.removeValue(forKey: $0)
      }
    }
    
  }
    
  public subscript(key: HashKey) -> Entity.EntityID? {
    get {
      backing[key].map(Entity.EntityID.init)
    }
    set {
      backing[key] = newValue?.any
    }
  }
  
}
